package jp.gr.java_conf.matsubara.expresser;

import jp.gr.java_conf.matsubara.expresser.exception.ExpException;
import jp.gr.java_conf.matsubara.expresser.exception.ExpTypeError;
import jp.gr.java_conf.matsubara.expresser.type.ExpBoolean;
import jp.gr.java_conf.matsubara.expresser.type.ExpNumber;
import jp.gr.java_conf.matsubara.expresser.type.ExpNumberBase;
import jp.gr.java_conf.matsubara.expresser.type.ExpString;
import jp.gr.java_conf.matsubara.expresser.type.IExpObject;
import jp.gr.java_conf.matsubara.expresser.type.IFunction;
import jp.gr.java_conf.matsubara.expresser.type.JavaClass;
import jp.gr.java_conf.matsubara.expresser.type.JavaObject;
import jp.gr.java_conf.matsubara.expresser.type.JavaObjectMember;
import jp.gr.java_conf.matsubara.expresser.type._Boolean;
import jp.gr.java_conf.matsubara.expresser.type._Double;
import jp.gr.java_conf.matsubara.expresser.type._String;

/**
 * ^ϊ̓K肵܂B
 * 
 * 쐬: 2005/05/22
 * 
 * @author m.matsubara
 */

public class ExpTypeConverter implements ITypeConverter {
    
    /**
     * v~eBuǂ肵܂B
     * @param value 肷l
     * @return v~eBu̎ true
     */
    public boolean isPrimitive(Object value) {
        if (value instanceof _Boolean)
            return true;
        else if (value instanceof ExpNumberBase)
            return true;
        else if (value instanceof _String)
            return true;
        return false;
    }
    

    /* ( Javadoc)
     * @see jp.gr.java_conf.matsubara.expresser.ITypeConverter#toPrimitive(RuntimeData, java.lang.Object)
     */
    public IExpObject toPrimitive(RuntimeData runtime, IExpObject value) throws ExpException {
        if (value == Expresser.UNDEFINED || value == null)
            return value;
        else if (value instanceof _Boolean)
            return value;
        else if (value instanceof ExpNumberBase)
            return value;
        else if (value instanceof _String)
            return value;
        else if (value instanceof JavaObject)
            return new _String(value.toString());
        else if (value instanceof JavaClass)
            return new _String(value.toString());
        else if (value instanceof JavaObjectMember)
            return new _String(value.toString());
        else if (value instanceof IExpObject) {
            //  toString() ł܂ϊł邩
            IExpObject object = (IExpObject)value;
            IExpObject property = object.getProperty(StringToId.toId("toString"));
            if (property instanceof IFunction) {
                IFunction function = (IFunction)property;
                IExpObject newValue = function.callFunction(runtime, object, new IExpObject[] {});
                if (isPrimitive(newValue))
                    return newValue;
            }
            //  valueOf() ŕϊł邩
            property = object.getProperty(StringToId.toId("valueOf"));
            if (property instanceof IFunction) {
                IFunction function = (IFunction)property;
                IExpObject newValue = function.callFunction(runtime, object, new IExpObject[] {});
                if (isPrimitive(newValue))
                    return newValue;
            }
        }
        throw new ExpTypeError(Expresser.getResString("ConvertError_Primitive"), -1);
    }
    
    /* ( Javadoc)
     * @see jp.gr.java_conf.matsubara.expresser.ITypeConverter#toBoolean(RuntimeData runtime, java.lang.Object)
     */
    public _Boolean toBoolean(RuntimeData runtime, IExpObject value) throws ExpException {
        if (value == Expresser.UNDEFINED || value == null)
            return new _Boolean(false);
        else if (value.getClass() == _Boolean.class)
            return (_Boolean)value;
        else if (value instanceof ExpNumberBase) {
            double doubleValue = ((ExpNumberBase)value).doubleValue();
            return new _Boolean(doubleValue != 0 && Double.isNaN(doubleValue) == false);
        }
        else if (value instanceof _String)
            if (value.toString().equals(""))
                return new _Boolean(false);
            else
                return new _Boolean(true);
        else if (value instanceof IExpObject)
            //  (new Boolean(false)) Ro[g true Ԃ܂ ECMA-262 ̎dl
            return new _Boolean(true);
        else
            return new _Boolean(true);
    }
    
    /* ( Javadoc)
     * @see jp.gr.java_conf.matsubara.expresser.ITypeConverter#toNumber(RuntimeData runtime, java.lang.Object)
     */
    public _Double toNumber(RuntimeData runtime, IExpObject value) throws ExpException {
        if (value == null)
            return new _Double(+0);
        else if (value == Expresser.UNDEFINED)
            return new _Double(Double.NaN);
        else if (value instanceof _Boolean)
            return new _Double(((_Boolean)value).booleanValue() ? 1 : 0);
        else if (value instanceof ExpNumberBase)
            return new _Double(((ExpNumberBase)value).doubleValue());
        else if (value instanceof _String)
            try {
                String string = ((_String)value).toString();
                return Expresser.toValue(string.toCharArray(), 0, string.length());
            } catch (Exception e) {
                return new _Double(0);
            }
        else if (value instanceof IExpObject) {
            value = toPrimitive(runtime, value);
            return toNumber(runtime, value);
        }
        else
            throw new ExpTypeError(Expresser.getResString("ConvertError_Number"), -1);
    }

    /* ( Javadoc)
     * @see jp.gr.java_conf.matsubara.expresser.ITypeConverter#toString(RuntimeData runtime, java.lang.Object)
     */
    public _String toString(RuntimeData runtime, IExpObject value) throws ExpException {
        if (value == Expresser.UNDEFINED)
            return new _String("undefined");
        else if (value == null)
            return new _String("null");
        else if (value instanceof _Boolean)
            return new _String(value.toString());
        else if (value instanceof ExpNumberBase)
            return new _String(value.toString());
        else if (value instanceof _String)
            return (_String)value;
        else if (value instanceof IExpObject)
            return toString(runtime, toPrimitive(runtime, value));
        else
            return new _String(value.toString());
    }

    /* ( Javadoc)
     * @see jp.gr.java_conf.matsubara.expresser.ITypeConverter#toObject(RuntimeData runtime, java.lang.Object)
     */
    public IExpObject toObject(RuntimeData runtime, IExpObject value) throws ExpException {
        if (value == Expresser.UNDEFINED || value == null)
            throw new ExpTypeError(Expresser.getResString("ConvertError_Object"), -1);
        else if (value instanceof _Boolean)
            return new ExpBoolean(((_Boolean)value).booleanValue());
        else if (value instanceof ExpNumberBase)
            return new ExpNumber(((ExpNumberBase)value).doubleValue());
        else if (value instanceof _String)
            return new ExpString(((_String)value).toString());
        else if (value instanceof IExpObject) {
            return (IExpObject)value;
        }
        else
            throw new ExpTypeError(Expresser.getResString("ConvertError_Object"), -1);
    }
}
