package jp.gr.java_conf.matsubara.expresser.operatorset;

import jp.gr.java_conf.matsubara.expresser.*;
import jp.gr.java_conf.matsubara.expresser.exception.ExpException;
import jp.gr.java_conf.matsubara.expresser.type.*;

/**
 * ^ɑΉ鉉ZqZbgłB<br/>
 * <br/>
 * 쐬: 2004/12/26
 * 
 * @author m.matsubara
 */
public class IntegerOperatorSet implements IOperatorSet {
    public static final IntegerOperatorSet operatorSet = new IntegerOperatorSet();

    /*
     *  ( Javadoc)
     * @see jp.gr.java_conf.matsubara.expresser.operatorset.IOperatorSet#targetType()
     */
    public Class[] targetType() {
        Class[] result = { _Integer.class };
        
        return result;
    }

    /*
     *  ( Javadoc)
     * @see jp.gr.java_conf.matsubara.expresser.operatorset.IOperatorSet#operation(int, java.lang.Object, java.lang.Object, jp.gr.java_conf.matsubara.expresser.RuntimeData)
     */
    public IExpObject operation(int operator, IExpObject operand1, IExpObject operand2, RuntimeData runtime) throws ExpException {
        switch (operator) {
        case OperatorData.OP_CMP_EQ_STRICT:
        	try {
        		return new _Boolean(operand1.equals(operand2));
        	} catch (ClassCastException e) {
        		//  f[^^Ȃ
        		return new _Boolean(false);
        	}
        case OperatorData.OP_CMP_NE_STRICT:
        	try {
        		return new _Boolean(! operand1.equals(operand2));
        	} catch (ClassCastException e) {
        		//  f[^^Ȃ
        		return new _Boolean(false);
        	}
        default:
	        if (!(operand2 instanceof _Integer)) {
                operand2 = runtime.getTypeConverter().toPrimitive(runtime, operand2);
                if (operand2 instanceof ExpNumberBase)
                    return DoubleOperatorSet.operatorSet.operation(operator, operand1, operand2, runtime);
                else if (operand2 instanceof _String)
                    return StringOperatorSet.operatorSet.operation(operator, operand1, operand2, runtime);
                else if (operand2 instanceof _Boolean)
                    operand2 = runtime.getTypeConverter().toNumber(runtime, operand2);
                else
                    return _DefaultOperatorSet.operatorSet.operation(operator, operand1, operand2, runtime);
	        }
	
	        int operand1value = (operand1 != null) ? ((ExpNumberBase)operand1).intValue() : 0;    //  IyhPnull̂Ƃ
	        int operand2value = ((ExpNumberBase)operand2).intValue();
	        switch (operator) {
	        //  rZq
	        case OperatorData.OP_CMP_EQ:
	            return new _Boolean(operand1value == operand2value); 
	        case OperatorData.OP_CMP_NE:
	            return new _Boolean(operand1value != operand2value); 
	        case OperatorData.OP_CMP_GE:
	            return new _Boolean(operand1value >= operand2value); 
	        case OperatorData.OP_CMP_GT:
	            return new _Boolean(operand1value > operand2value);
	        case OperatorData.OP_CMP_LE:
	            return new _Boolean(operand1value <= operand2value);
	        case OperatorData.OP_CMP_LT:
	            return new _Boolean(operand1value < operand2value);
	        //  ZpZq
	        case OperatorData.OP_ARI_ADD:
	            return new _Integer(operand1value + operand2value);
	        case OperatorData.OP_ARI_SUB:
	            return new _Integer(operand1value - operand2value);
	        case OperatorData.OP_ARI_MUL:
	            return new _Integer(operand1value * operand2value);
	        case OperatorData.OP_ARI_DIV:
	            if (operand2value == 0) {
	                if (operand1value == 0)
	                    return new _Double(Double.NaN);
	                else if (operand1value > 0)
	                    return new _Double(Double.POSITIVE_INFINITY);
	                else
	                    return new _Double(Double.NEGATIVE_INFINITY);
	            }
	            return new _Integer(operand1value / operand2value);
	        case OperatorData.OP_ARI_REM:
	            if (operand2value == 0)
	                return new _Double(Double.NaN);
	            return new _Integer(operand1value % operand2value);
	        //  
	        case OperatorData.OP_PLUS:
	            return operand2; 
	        //  
	        case OperatorData.OP_MINUS:
	            return new _Integer(0 - operand2value);
	        //  rbgZq
	        case OperatorData.OP_BIT_NOT:
	            return new _Integer(~ operand2value); 
	        case OperatorData.OP_BIT_AND:
	            return new _Integer(operand1value & operand2value); 
	        case OperatorData.OP_BIT_XOR:
	            return new _Integer(operand1value ^ operand2value); 
	        case OperatorData.OP_BIT_OR:
	            return new _Integer(operand1value | operand2value);
	        //  VtgZq
	        case OperatorData.OP_SFT_LEFT:
	            return new _Integer(operand1value << operand2value); 
	        case OperatorData.OP_SFT_RIGHT:
	            return new _Integer(operand1value >> operand2value); 
	        case OperatorData.OP_SFT_RIGHT_UNSIGNED:
	            return new _Integer(operand1value >>> operand2value); 
	        
	        default:
	            return _DefaultOperatorSet.operatorSet.operation(operator, operand1, operand2, runtime);
	        }
        }
    }

}
