package jp.gr.java_conf.matsubara.expresser.parsedexpression;

import jp.gr.java_conf.matsubara.expresser.Expresser;
import jp.gr.java_conf.matsubara.expresser.IVariable;
import jp.gr.java_conf.matsubara.expresser.OperatorData;
import jp.gr.java_conf.matsubara.expresser.RuntimeData;
import jp.gr.java_conf.matsubara.expresser.exception.ExpException;
import jp.gr.java_conf.matsubara.expresser.operatorset.IOperatorSet;
import jp.gr.java_conf.matsubara.expresser.type.IExpObject;

/**
 * \͍ς݂̌vZłB<br/>
 * <br/>
 * 쐬: 2005/02/05
 * 
 * @author m.matsubara
 */
public class ParsedExpression implements IParsedExpression {
    /**  IyhPAIyh\܂BIyĥȂPZq null ƂȂ܂ */
    private IParsedExpression operand1Expression;
    /**  IyhQAEIyh\܂BEIyĥȂPZq null ƂȂ܂ */
    private IParsedExpression operand2Expression;
    /**  Zq\܂ */
    private OperatorData operatorData;
    /**  \[Xł̉Zqʒu\܂ */
    private int operatorIdx;
    /**  Zł */
    private Expresser expresser;
    /**  O񉉎Z̕]sƂ̎Iyh̃NXł */
    private Class lastPrimaryOperandClass = null;
    /**  O񉉎Z̕]sƂ̉ZqZbgł */
    private IOperatorSet lastOperatorSet = null;
    
    public ParsedExpression(IParsedExpression operand1Expression, IParsedExpression operand2Expression, OperatorData operatorData, int operatorIdx, Expresser expresser) throws ExpException {
        this.operand1Expression = operand1Expression;
        this.operand2Expression = operand2Expression;
        this.operatorData = operatorData;
        this.operatorIdx = operatorIdx;
        this.expresser = expresser;
    }

    /*
     *  ( Javadoc)
     * @see jp.gr.java_conf.matsubara.expresser.IParsedExpression#isUsingVariable(Integer id)
     */
    public int isUsingVariable(Integer id) throws ExpException {
        int result = 0;
        if (operand1Expression != null)
            result += operand1Expression.isUsingVariable(id);
        if (operand2Expression != null)
            result += operand2Expression.isUsingVariable(id);
        return result;
    }
    
    /*
     *  ( Javadoc)
     * @see jp.gr.java_conf.matsubara.expresser.parsedexpression.IParsedExpression#evaluate(jp.gr.java_conf.matsubara.expresser.RuntimeData)
     */
    public IExpObject evaluate(RuntimeData runtime) throws ExpException {
        IExpObject operand1 = null;
        IExpObject operand2 = null;
        try {
            if (this.operand1Expression != null) {
                if (operatorData.isLeftVariable() == false)
                    operand1 = this.operand1Expression.evaluate(runtime);
                else
                    operand1 = this.operand1Expression.evalRef(runtime);
            }
            if (this.operand2Expression != null) {
                if (operatorData.isRightVariable() == false)
                    operand2 = this.operand2Expression.evaluate(runtime);
                else
                    operand2 = this.operand2Expression.evalRef(runtime);
            }

            Object primaryOperand = ((operand1 != null) && (operand1 != null)) ? operand1 : operand2;        //  ̂ƂȂ鉉ZΏ
            Class primaryOperandClass = primaryOperand != null ? primaryOperand.getClass() : Object.class;
            
            //  vZΏۂOƓ^AgB
            if (primaryOperandClass == lastPrimaryOperandClass)
                return lastOperatorSet.operation(operatorData.getOperator(), operand1, operand2, runtime);
            
            //  expresserpđΉ鉉Zq
            IOperatorSet opeSet = expresser.searchOperatorSet(primaryOperand);
            if (opeSet != null) {
                IExpObject value = opeSet.operation(operatorData.getOperator(), operand1, operand2, runtime);
                if (value != Expresser.UNSUPPORTED_OPERATOR) {
                    lastPrimaryOperandClass = primaryOperandClass;
                    lastOperatorSet = opeSet;
                    return value;
                }    
            }
        } catch (ExpException ee) {
            //  position ێȂO⑫ApositionĐݒ肵ăX[
            if (ee.getPosition() < 0)
                ee.setPosition(this.operatorIdx);
               throw ee;        //  Lb`OʒuێĂȂꍇAZqʒuG[ʒuƂčăX[
        }
        throw new ExpException(Expresser.getResString("UnsupportedOperator", operand1, operatorData.toString(), operand2), operatorIdx);
    }
    
    /* (non-Javadoc)
     * @see jp.gr.java_conf.matsubara.expresser.parsedexpression.IParsedExpression#evalRef(jp.gr.java_conf.matsubara.expresser.RuntimeData)
     */
    public IVariable evalRef(RuntimeData runtime) throws ExpException {
        throw new ExpException(Expresser.getResString("NotSubstituteToConstant"), -1);
    }
    
    /* (non-Javadoc)
     * @see jp.gr.java_conf.matsubara.expresser.parsedexpression.IParsedExpression#evalTargetObject(jp.gr.java_conf.matsubara.expresser.RuntimeData)
     */
    public IExpObject evalTargetObject(RuntimeData runtime) throws ExpException {
        return Expresser.UNDEFINED;
    }
    
    /* (non-Javadoc)
     * @see jp.gr.java_conf.matsubara.expresser.parsedexpression.IParsedExpression#isConst()
     */
    public boolean isConst() {
        if (operand1Expression != null && operand1Expression.isConst() == false)
            return false;
        if (operand2Expression != null && operand2Expression.isConst() == false)
            return false;
        return true;
    }
    
    public OperatorData getOperatorData() {
        return operatorData;
    }
    
    public Object getOperand1() {
        return operand1Expression;
    }

    public Object getOperand2() {
        return operand2Expression;
    }
}
