package jp.gr.java_conf.matsubara.expresser.scope;

import jp.gr.java_conf.matsubara.expresser.Expresser;
import jp.gr.java_conf.matsubara.expresser.IVariable;
import jp.gr.java_conf.matsubara.expresser.exception.ExpException;
import jp.gr.java_conf.matsubara.expresser.type.ExpObjectMember;
import jp.gr.java_conf.matsubara.expresser.type.IExpObject;

/**
 * XR[v
 * 
 * 쐬: 2005/06/04
 * 
 * @author m.matsubara
 */

public class ObjectScope implements IScope {
    IExpObject object;  //  XR[v̌ƂȂIuWFNg
    private IScope scopeChain = null;
    
    public ObjectScope(IExpObject object) {
        this.object = object;
    }

	/* ( Javadoc)
	 * @see jp.gr.java_conf.matsubara.expresser.IScope#getValue(java.lang.Integer)
	 */
	public IExpObject getValue(Integer id) throws ExpException {
        if (object.isProperty(id, true))
        	return object.getProperty(id);
        else if (scopeChain != null)
            return getValue(id);
        else
        	throw new ExpException(Expresser.getResString("UndefinedVariable"), -1);
        
	}
    
    /* ( Javadoc)
     * @see jp.gr.java_conf.matsubara.expresser.IScope#getValue(java.lang.Integer, int)
     */
    public IExpObject getValue(Integer id, int offset) throws ExpException {
        if (object.isProperty(id, true))
            return object.getProperty(id);
        else if (scopeChain != null)
            return getValue(id, offset);
        else
            throw new ExpException(Expresser.getResString("UndefinedVariable"), -1);
    }
    
    /* ( Javadoc)
     * @see jp.gr.java_conf.matsubara.expresser.IScope#setValue(java.lang.Integer, java.lang.Object)
     */
    public void setValue(Integer id, IExpObject value) throws ExpException {
        if (object.isProperty(id, true))
        	object.putProperty(id, value);
        else if (scopeChain != null)
            setValue(id, value);
        else
            throw new ExpException(Expresser.getResString("UndefinedVariable"), -1);
    }

    /* ( Javadoc)
     * @see jp.gr.java_conf.matsubara.expresser.IScope#setValue(java.lang.Integer, java.lang.Object)
     */
    public void setValue(Integer id, int offset, IExpObject value) throws ExpException {
        if (object.isProperty(id, true))
            object.putProperty(id, value);
        else if (scopeChain != null)
            setValue(id, offset, value);
        else
            throw new ExpException(Expresser.getResString("UndefinedVariable"), -1);
    }

	/* ( Javadoc)
	 * @see jp.gr.java_conf.matsubara.expresser.IScope#getVariable(java.lang.Integer)
	 */
	public IVariable getVariable(Integer id) throws ExpException {
		return new ExpObjectMember(object, id);
	}
    
    /* ( Javadoc)
     * @see jp.gr.java_conf.matsubara.expresser.IScope#getVariable(java.lang.Integer)
     */
    public IVariable getVariable(Integer id, int offset) throws ExpException {
        if (object.isProperty(id, true))
        	return new ExpObjectMember(object, id);
        else if (scopeChain != null)
            return getVariable(id, offset);
        else
            throw new ExpException(Expresser.getResString("UndefinedVariable"), -1);
        
    }
    
	/* ( Javadoc)
	 * @see jp.gr.java_conf.matsubara.expresser.IScope#defineVariable(java.lang.Integer)
	 */
    public IVariable defineVariable(Integer id) throws ExpException {
    	object.putProperty(id, Expresser.UNDEFINED);
        return getVariable(id);
    }

    /* ( Javadoc)
     * @see jp.gr.java_conf.matsubara.expresser.IScope#setScopeChain(jp.gr.java_conf.matsubara.expresser.IScope)
     */
    public void setScopeChain(IScope nextScope) {
        this.scopeChain = nextScope;
    }

    /* ( Javadoc)
     * @see jp.gr.java_conf.matsubara.expresser.IScope#getScopeChain()
     */
    public IScope getScopeChain() {
        return scopeChain;
    }
    
    /* ( Javadoc)
     * @see jp.gr.java_conf.matsubara.expresser.IScope#getOffset(java.lang.Integer)
     */
    public int getOffset(Integer id) {
    	return 0;
    }
    
}
