package jp.gr.java_conf.matsubara.interpreter;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.ResourceBundle;

import jp.gr.java_conf.matsubara.expresser.Expresser;
import jp.gr.java_conf.matsubara.expresser.ITypeConverter;
import jp.gr.java_conf.matsubara.expresser.OperatorData;
import jp.gr.java_conf.matsubara.expresser.RuntimeData;
import jp.gr.java_conf.matsubara.expresser.StringToId;
import jp.gr.java_conf.matsubara.expresser.exception.ExpException;
import jp.gr.java_conf.matsubara.expresser.type.IExpObject;
import jp.gr.java_conf.matsubara.interpreter.ecmascript.operatorparser.FunctionOperatorParser;
import jp.gr.java_conf.matsubara.interpreter.ecmascript.parser.BreakStatementParser;
import jp.gr.java_conf.matsubara.interpreter.ecmascript.parser.ContinueStatementParser;
import jp.gr.java_conf.matsubara.interpreter.ecmascript.parser.DoWhileStatementParser;
import jp.gr.java_conf.matsubara.interpreter.ecmascript.parser.ExpressionStatementParser;
import jp.gr.java_conf.matsubara.interpreter.ecmascript.parser.ForStatementParser;
import jp.gr.java_conf.matsubara.interpreter.ecmascript.parser.FunctionStatementParser;
import jp.gr.java_conf.matsubara.interpreter.ecmascript.parser.IStatementParser;
import jp.gr.java_conf.matsubara.interpreter.ecmascript.parser.IfStatementParser;
import jp.gr.java_conf.matsubara.interpreter.ecmascript.parser.ReturnStatementParser;
import jp.gr.java_conf.matsubara.interpreter.ecmascript.parser.SwitchStatementParser;
import jp.gr.java_conf.matsubara.interpreter.ecmascript.parser.SynchronizedStatementParser;
import jp.gr.java_conf.matsubara.interpreter.ecmascript.parser.ThrowStatementParser;
import jp.gr.java_conf.matsubara.interpreter.ecmascript.parser.TryStatementParser;
import jp.gr.java_conf.matsubara.interpreter.ecmascript.parser.VarStatementParser;
import jp.gr.java_conf.matsubara.interpreter.ecmascript.parser.WhileStatementParser;
import jp.gr.java_conf.matsubara.interpreter.ecmascript.parser.WithStatementParser;
import jp.gr.java_conf.matsubara.interpreter.ecmascript.type.EcmaFunction;
import jp.gr.java_conf.matsubara.interpreter.ecmascript.type.constructor.Function_constructor;
import jp.gr.java_conf.matsubara.interpreter.ecmascript.type.property.Global__eval;
import jp.gr.java_conf.matsubara.interpreter.statement.BlockStatement;
import jp.gr.java_conf.matsubara.interpreter.statement.IStatement;
import jp.gr.java_conf.matsubara.resource.DynaResourceBundle;




/**
 * ܂̂ƂU Ignis interpreter<br/>
 * ߍݗprXNvg<br/>
 * 
 * 쐬: 2005/01/15
 * 
 * @author m.matsubara
 */
public class ScriptEngine {
    public static String RESOURCE_NAME = "jp.gr.java_conf.matsubara.interpreter.ScriptEngineRes";
    public static DynaResourceBundle resource = null;

    /**  ZivZ]NXj */
    private Expresser expresser;
    
    /** eXe[gg̃p[T[o^܂B */
    private HashMap statementParserMap = new HashMap();
    /** ̃L[[hŎn܂ȂXe[ggp[T[łBvZp[T[łB */
    private IStatementParser defaultParseStatement;
    
    /** \̈ꗗ */
    public final String[] RESERVED_WORDS = new String[] {
            "as", "break", "case", "catch", "class", "const", "continue", "default", "delete", "do", "else", "export", "extends", "finally", "for", "function", "if", "import", "in", "instanceof", "is", "namespace", "new", "package", "private", "public", "return", "super", "switch", "throw", "try", "typeof", "use", "var", "void", "while", "with",
            "abstract", "debugger", "enum", "goto", "implements", "interface", "native", "protected", "synchronized", "throws", "transient", "volatile",
    };
    
    /** 
     * XNvgt@Cǂݍލۂ̃LN^ZbgłB<br/>
     * null ̎̓VXeɊÂLN^Zbgp܂B  
     */
    private String scriptCharacterSet = null;
    
    /**
     * XNvgGW\z܂B
     * @throws Exception
     */
    public ScriptEngine() throws ExpException {
        //  vZIuWFNg̍쐬
        expresser = new Expresser();
        //  Zq̒ǉ
        int priority = expresser.findOpeData(OperatorData.OP_LOG_NOT).getPriority();  //  ! Zq̗Dx擾
        expresser.addOpeData(new OperatorData("function", OperatorData.OP_EXTERNAL_PARSED, priority, OperatorData.OP_CLS_FRONT_UNITY, false, false, new FunctionOperatorParser(this)));
        
        //  O[oȃvpeBi֐j̒ǉ
        EcmaFunction.constructor = new Function_constructor(this);

        RuntimeData runtime = expresser.getDefaultRuntimeData();
        runtime.setConst(Global__eval.ID, Global__eval.getInstance());
        runtime.setConst(StringToId.toId("Function"), EcmaFunction.constructor);

        //  p[T[̓o^
        //  ECMAScript ݊p[T[
        addStatementParser(new BreakStatementParser());
        addStatementParser(new ContinueStatementParser());
        addStatementParser(new DoWhileStatementParser());
        addStatementParser(new ReturnStatementParser());
        addStatementParser(new ForStatementParser());
        addStatementParser(new FunctionStatementParser());
        addStatementParser(new IfStatementParser());
        addStatementParser(new SwitchStatementParser());
        addStatementParser(new ThrowStatementParser());
        addStatementParser(new TryStatementParser());
        addStatementParser(new VarStatementParser());
        addStatementParser(new WhileStatementParser());
        addStatementParser(new WithStatementParser());
        setDefaultStatementParser(new ExpressionStatementParser());

        //  Ǝgp[T[
        addStatementParser(new SynchronizedStatementParser());
        
        for (int idx = 0; idx < RESERVED_WORDS.length; idx++)
            expresser.addReservedWord(RESERVED_WORDS[idx]);
    }
    
    /**
     * expresser 擾܂B
     * @return expresser
     */
    public Expresser getExpresser() {
        return expresser;
    }
    
    /**
     * \͗pXe[ggo^܂B
     * @param parser p[T[
     */
    public void addStatementParser(IStatementParser parser) {
        statementParserMap.put(parser.targetStatement(), parser);
        expresser.addReservedWord(parser.targetStatement());
    }
    
    /**
     * ftHg̃Xe[ggp[T[łB
     * 
     * @param defaultStatement defaultStatement ݒ
     */
    public void setDefaultStatementParser(IStatementParser defaultStatement) {
        this.defaultParseStatement = defaultStatement;
    }

    /**
     * Xe[ggp[T[擾܂B
     * @param statement Xe[gg
     * @return ͂Ɏgpp[T[
     */
    public IStatementParser getStatementParser(String statement) {
        IStatementParser result = (IStatementParser)statementParserMap.get(statement);
        if (result != null)
            return result;
        else
            return defaultParseStatement;
    }
    
    /**
     * XNvg̉͂܂B
     * @param script XNvg
     * @return \͂ꂽXNvg
     * @throws ExpException
     */
    public ParsedScript parse(String script) throws ExpException {
        char[] scriptChars = script.toCharArray();
        return parse(scriptChars, 0, scriptChars.length);
    }
    
    /**
     * XNvg̉͂܂B
     * @param script XNvg
     * @param begin Jnʒu
     * @param end Iʒu
     * @return \͂ꂽXNvg
     * @throws Exception
     */
    public ParsedScript parse(char[] script, int begin, int end) throws ExpException {
        BlockStatement parsedStatement = new BlockStatement();
        while (begin < end) {
            begin = parseStatement1(parsedStatement, script, begin, end);
        }
        ParsedScript result = new ParsedScript(parsedStatement, expresser.getDefaultRuntimeData());
        return result;
    }

    /**
     * Xe[gg̉͂܂B<br/>
     * parse() \bh̒IɌĂяo܂BʏĂяoKv͂܂B
     * @param script XNvg
     * @param begin Jnʒu
     * @param end Iʒu
     * @return \͂ꂽXNvg
     * @throws Exception
     */
    public IStatement parseStatement(char[] script, int begin, int end) throws ExpException {
        BlockStatement parsedStatement = new BlockStatement();
        while (begin < end) {
            begin = parseStatement1(parsedStatement, script, begin, end);
        }
        return parsedStatement;
    }
    
    /**
     * Xe[gg̉͂܂(PXe[gg܂͂PubNł)<br/>
     * parse() \bh̒IɌĂяo܂BʏĂяoKv͂܂B
     * @param script XNvg(Xe[gg)
     * @param begin Jnʒu
     * @param end Iʒu
     */
    public int parseStatement1(BlockStatement blockStatement, char[] script, int begin, int end) throws ExpException {
        //  Ő󔒏()
        begin = Expresser.skipWhitespace(script, begin, end);
        if (begin >= end)
            return begin;

        if (script[begin] == '{') {
            //  ubNXe[gg
            int blockBegin = begin;
            begin = Expresser.skipParenthesis(script, begin, end);
            if (script[begin - 1] == '}') {
                blockStatement.add(parseStatement(script, blockBegin + 1, begin - 1));
            }
        } else {
            String statement = expresser.nextToken(script, begin, end);
            int labelIdx = begin + statement.length();
            labelIdx = Expresser.skipWhitespace(script, labelIdx, end);
            if (labelIdx >= end)
                return labelIdx;
            if (script[labelIdx] == ':') {
                //  x
                Integer labelId = StringToId.toId(statement);
                begin = labelIdx + 1;
                begin = Expresser.skipWhitespace(script, begin, end);

                statement = expresser.nextToken(script, begin, end);
                IStatementParser statementParser = getStatementParser(statement);
                begin = statementParser.parseStatement(this, blockStatement, statement, script, begin, end, labelId);
            } else {
                //  ʏ̃Xe[gg
                IStatementParser statementParser = getStatementParser(statement);
                begin = statementParser.parseStatement(this, blockStatement, statement, script, begin, end, null);
            }
        }
        return begin;
    }

    
    /**
     * ̃Xe[gg̏I܂œǂݔ΂܂B
     * @param script XNvg
     * @param begin Jnʒu
     * @param end Iʒu
     * @return Z~R̈ʒu+1
     * @throws ExpException
     */
    public static int skipStatement(char[] script, int begin, int end) throws ExpException {
        while (begin < end) {
            char ch = script[begin];
            if (ch == '{' || ch == '(' || ch == '[')
                begin = Expresser.skipParenthesis(script, begin, end);
            else if (ch == '\"' || ch == '\'')
                begin = Expresser.skipString(script, begin, end);
            else if (ch == ';')
                return begin + 1;
            else {
                begin++;
            }
        }
        throw new ExpException(ScriptEngine.getResString("UnmatchParenthesis"), begin);
    }
    
    /**
     * ̎̏I܂œǂݔ΂܂B<br/>
     * J}ł̏IƔf邱ƂɒӂȂ΂Ȃ܂B
     * @param script XNvg
     * @param begin Jnʒu
     * @param end Iʒu
     * @return Z~R܂̓J}̈ʒu+1
     * @throws ExpException
     */
    public static int skipExpression(char[] script, int begin, int end) throws ExpException {
        while (begin < end) {
            char ch = script[begin];
            if (ch == '{' || ch == '(' || ch == '[')
                begin = Expresser.skipParenthesis(script, begin, end);
            else if (ch == '\"' || ch == '\'')
                begin = Expresser.skipString(script, begin, end);
            else if (ch == ';')
                return begin + 1;
            else {
                begin++;
            }
        }
        throw new ExpException(ScriptEngine.getResString("UnmatchParenthesis"), begin);
    }
    
    
    /**
     * XNvgs܂B<br/>
     * VOXbhpłB<br/>
     * }`Xbhł parse()\bh̖߂lParsedScriptexecute()\bhAexpresser̃ftHg^C쐬V^Cf[^ƂēnČĂяoKv܂B
     * @param script XNvg
     * @return s
     * @throws ExpException
     */
    public Object execute(String script) throws ExpException {
        ParsedScript parsedScript = parse(script);
        return parsedScript.execute();
    }
    
    /**
     * Rg폜܂B
     * @param script Rgt̃XNvg
     * @param begin Jnʒu
     * @param end Iʒu
     * @throws ExpException
     */
    public static void deleteComents(char[] script, int begin, int end) throws ExpException {
        for (int idx = begin; idx < end - 1; idx++) {
            char ch = script[idx];
            if (ch == '/') {
                char ch2 = script[idx + 1];
                if (ch2 == '/') {
                    while (idx < end && script[idx] != '\n') {
                        script[idx] = ' ';
                        idx++;
                    }
                } else if (ch2 == '*') {
                    script[idx] = ' ';
                    idx++;
                    while (idx < end - 1 && !(script[idx] == '*' && script[idx + 1] == '/')) {
                        if (script[idx] != '\n')
                            script[idx] = ' ';
                        idx++;
                    }
                    if (idx >= end - 1)
                        throw new ExpException(ScriptEngine.getResString("CommentNotEnded"), idx);
                    script[idx] = ' ';
                    script[idx + 1] = ' ';
                }
            }
        }
    }
    
    /**
     * \[X͂܂B
     * @param scriptsChars ׂ͂\[X
     * @return ͍ς݃XNvg
     * @throws ExpException
     */
    public ParsedScript parseScript(char[] scriptsChars) throws ExpException {
        deleteComents(scriptsChars, 0, scriptsChars.length);
        //  
        ParsedScript parsedScript = parse(scriptsChars, 0, scriptsChars.length);
        parsedScript.setSource(scriptsChars);
        return parsedScript;
    }
    
    
    /**
     * Xg[\[Xǂݍ݂܂B 
     * @param stream ǂݍ݃Xg[
     * @param charsetName LN^Zbg
     * @return Xg[ǂݍ܂ꂽ\[X
     * @throws ExpException
     * @throws IOException
     */
    public char[] loadFromStream(InputStream stream, String charsetName) throws ExpException, IOException {
        //  Xg[̓ǂݍݏ
        BufferedReader fileReader;
        if (charsetName != null)
            fileReader = new BufferedReader(new InputStreamReader(stream, charsetName));
        else
            fileReader = new BufferedReader(new InputStreamReader(stream));
        StringBuffer scriptBuffer = new StringBuffer();
        //  Xg[̓ǂݍ
        while (true) {
            String line = fileReader.readLine();
            if (line == null)
                break;
            scriptBuffer.append(line);
            scriptBuffer.append('\n');
        }
        //  ͏
        char[] scriptsChars = new char[scriptBuffer.length()]; 
        scriptBuffer.getChars(0, scriptBuffer.length(), scriptsChars, 0);
        return scriptsChars;
    }

    /**
     * t@C\[Xǂݍ݂܂B
     * @param scriptFileName ͂t@C
     * @param charsetName LN^Zbg
     * @return t@Cǂݍ܂ꂽ\[X
     * @throws ExpException
     * @throws FileNotFoundException
     * @throws IOException
     */
    public char[] loadFromFile(String scriptFileName, String charsetName) throws ExpException, FileNotFoundException, IOException {
        File file = new File(scriptFileName);
        InputStream stream = new FileInputStream(file);
        char[] result = loadFromStream(stream, charsetName);
        stream.close();
        return result;
    }

    /**
     * t@CXNvgǂݍ݁A͂s܂B
     * @param scriptFileName ͂t@C
     * @return ͍ς݃XNvg
     * @throws ExpException ͂Ɏsꍇ
     * @throws FileNotFoundException t@CȂꍇ
     * @throws IOException ǂݍ݂Ɏsꍇ
     */
    public char[] loadFromFile(String scriptFileName) throws ExpException, FileNotFoundException, IOException {
        File file = new File(scriptFileName);
        InputStream stream = new FileInputStream(file);
        char[] result = loadFromStream(stream, scriptCharacterSet);
        stream.close();
        return result;
    }
    

    /**
     * XNvgt@CǂݍŎs܂B
     * @param scriptFileName XNvgt@C
     * @param displayTime ԕ\s true
     * @throws ExpException
     * @throws FileNotFoundException
     * @throws IOException
     */
    public void runScriptFile(String scriptFileName, boolean displayTime) throws ExpException, FileNotFoundException, IOException {
        long startTime = System.currentTimeMillis();
        double scriptTime;
        ParsedScript parsedScript = null;
        char[] scriptChars = null;
        try {
            scriptChars = loadFromFile(scriptFileName);
            parsedScript = parseScript(scriptChars);
            if (displayTime) {
                scriptTime = (System.currentTimeMillis() - startTime) / 1000.0;
                System.out.println("XNvg\͎ : " + scriptTime + "s");
            }
            parsedScript.execute();
        } catch (ExpException ee) {
            int position = ee.getPosition();
            if (scriptChars != null && scriptChars.length > position) {
                int row = 1;
                int col = 1;
                for (int idx = 0; idx < position; idx++) {
                    if (scriptChars[idx] == '\n') {
                        row++;
                        col = 1;
                    }
                    else if (scriptChars[idx] == '\t') {
                        col += 4;   //֋X
                    }
                    else
                        col++;
                }
                System.err.println(ee.getMessage());
                System.err.println("  row : " + row);
                System.err.println("  col : " + col);
            } else {
                ee.printStackTrace();
            }
        }
        if (displayTime) {
            scriptTime = (System.currentTimeMillis() - startTime) / 1000.0;
            System.out.println("XNvgsv : " + scriptTime + "s");
        }
    }


    /**
     * vOɊւԂ܂B
     * @return vOɊւ
     */
    public static String aboutString() {
        return "Ignis interpreter version 0.3.0 (alpha edition)\n"
              + "  ECMAScript Language 3rd Edition based\n"
              + "  Copyright (C) 2005 m.matsubara";
    }
    
    
    /**
     * IbZ[WɑΉ\[Xoh擾܂
     * @return \[Xoh
     */
    public static DynaResourceBundle getResource() {
        if (resource == null)
            resource = new DynaResourceBundle(ResourceBundle.getBundle(RESOURCE_NAME));
        return resource;
    }
    
    /**
     * L[烍[PɑΉbZ[W擾܂
     * @param key L[
     * @return bZ[W
     */
    public static String getResString(String key) {
        return getResource().getString(key);
    }

    /**
     * L[烍[PɑΉbZ[W擾܂
     * @param key L[
     * @param params up[^̔z
     * @return bZ[W
     */
    public static String getResString(String key, Object[] params) {
        return getResource().getString(key, params);
    }

    /**
     * L[烍[PɑΉbZ[W擾܂
     * @param key L[
     * @param param1 up[^1
     * @return bZ[W
     */
    public static String getResString(String key, Object param1) {
        return getResource().getString(key, param1);
    }
    
    /**
     * L[烍[PɑΉbZ[W擾܂
     * @param key L[
     * @param param1 up[^1
     * @param param2 up[^2
     * @return bZ[W
     */
    public static String getResString(String key, Object param1, Object param2) {
        return getResource().getString(key, param1, param2);
    }

    /**
     * L[烍[PɑΉbZ[W擾܂
     * @param key L[
     * @param param1 up[^1
     * @param param2 up[^2
     * @param param3 up[^3
     * @return bZ[W
     */
    public static String getResString(String key, Object param1, Object param2, Object param3) {
        return getResource().getString(key, param1, param2, param3);
    }

    /**
     * L[烍[PɑΉbZ[W擾܂
     * @param key L[
     * @param param1 up[^1
     * @param param2 up[^2
     * @param param3 up[^3
     * @param param4 up[^4
     * @return bZ[W
     */
    public static String getResString(String key, Object param1, Object param2, Object param3, Object param4) {
        return getResource().getString(key, param1, param2, param3, param4);
    }

    /**
     * s|CgłB<br/>
     * R\[AvP[VƂĎs
     * @param argv XNvgt@C
     * @throws Exception
     */
    public static void main(String[] argv) throws Exception {
        ScriptEngine scriptEngine = new ScriptEngine();
        ParsedScript script = null;
        
        if (argv.length == 0) {
            //  Ȃł̋N
            System.out.println(aboutString());
            System.out.println();

            Expresser expresser = scriptEngine.getExpresser();
            RuntimeData runtime = expresser.getDefaultRuntimeData();
            ITypeConverter typeConverter = runtime.getTypeConverter();
            
            BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));
            System.out.print(">");
            System.out.flush();
            String statement = reader.readLine();
            while (statement != null && statement.equals("@exit") == false && statement.equals(".") == false) {
                try {
                    statement = Expresser.trim(statement) + " ";
                    if (statement.length() != 1) {
                        if (statement.startsWith("@run ")) {
                            String fileName = statement.substring(5).trim();
                            //long startTime = System.currentTimeMillis();
                            //double scriptTime = 0;
                            if (fileName.equals("") == false) {
                                //System.out.println("-----------------------------------------------------");
                                char[] scriptChars = scriptEngine.loadFromFile(fileName);
                                script = scriptEngine.parseScript(scriptChars);
                                //scriptTime = ((System.currentTimeMillis() - startTime) / 1000.0);
                                //System.out.println(ScriptEngine.getResString("ScriptParseTime", String.valueOf(scriptTime)));
                            }
                            if (script != null) {
                                script.execute(scriptEngine.getExpresser().getDefaultRuntimeData());
                                //scriptTime = (System.currentTimeMillis() - startTime) / 1000.0;
                                //System.out.println(ScriptEngine.getResString("ScriptTotalRunTime", String.valueOf(scriptTime)));
                            }
                        } else if (statement.startsWith("@start ")) {
                            String fileName = statement.substring(7).trim();
                            if (fileName.equals("") == false) {
                                char[] scriptChars = scriptEngine.loadFromFile(fileName);
                                script = scriptEngine.parseScript(scriptChars);
                            }
                            if (script != null) {
                                script.executeNewThread();
                                System.out.println(ScriptEngine.getResString("StartThread"));
                                
                            }
                        } else if (statement.startsWith("@load ")) {
                            String fileName = statement.substring(6).trim();
                            if (fileName.equals("") == false) {
                                char[] scriptChars = scriptEngine.loadFromFile(fileName);
                                script = scriptEngine.parseScript(scriptChars);
                            } else {
                                throw new ExpException(ScriptEngine.getResString("SpecifyFile"), 0);
                            }
                        } else if (statement.startsWith("@unload ")) {
                            script = null;
                        } else if (statement.startsWith("@mem ")) {
                            System.out.println("free memory  : " + Runtime.getRuntime().freeMemory());
                            System.out.println("total memory : " + Runtime.getRuntime().totalMemory());
//                            System.out.println("max memory   : " + Runtime.getRuntime().maxMemory());	//	JDK1.2Ɩ\bhc
                        } else if (statement.startsWith("@gc ")) {
                            System.gc();
                        } else if (statement.startsWith("@about")) {
                            System.out.println(aboutString());
                        } else {
                            try {
                                statement = statement.trim();
                                if ((statement.length() > 0) && (statement.charAt(statement.length() - 1) == ';'))
                                    statement = statement.substring(0, statement.length() - 1);
//                              ParsedScript parsedScript = scriptEngine.parse(statement);
//                              parsedScript.execute(expresser.getDefaultRuntimeData());
                                IExpObject object = expresser.evaluate(runtime, statement);
                                
                                if (object != Expresser.UNDEFINED) { //  undefined łȂꍇ̂ݕ\
                                    object = typeConverter.toString(runtime, object);
                                    System.out.println(" = " + object);
                                }
                            } catch (ExpException ee) {
                                System.out.println(statement);
                                int max = ee.getPosition();
                                for (int idx = 0; idx < max; idx++)
                                    System.out.print("-");
                                System.out.println("^");
                                System.out.println(ee.getMessage());
                            }
                        }
                    }
                } catch (ExpException ee) {
                    System.out.println(statement);
/*
                    int max = ee.getPosition();
                    for (int idx = 0; idx < max; idx++)
                        System.out.print("-");
                    System.out.println("^");
                    System.out.println(ee.getMessage());
*/
                    if (script != null) {
                        char[] source = script.getSource();
                        if (script != null) {
                            int nPosition = ee.getPosition();
                            int row = 1;
                            int col = 1;
                            for (int idx = 0; idx < nPosition; idx++) {
                                if (source[idx] == '\n') {
                                    row++;
                                    col = 1;
                                }
                                else if (source[idx] == '\t') {
                                    col += 4;   //֋X
                                }
                                else
                                    col++;
                            }
                            System.err.println(ee.getMessage());
                            System.err.println("  row : " + row);
                            System.err.println("  col : " + col);
                        }
                    } else {
                        ee.printStackTrace();
                    }
                }
                System.out.print(">");
                System.out.flush();
                statement = reader.readLine();
            }
        } else {
            //  ̋NiXNvgt@C̎sj
            int fileIdx = 0;
            while (fileIdx < argv.length) {
                if (argv[fileIdx].charAt(0) == '-') {
                    if (argv[fileIdx].equals("-charset")) {
                        if (argv.length >= fileIdx + 1)
                            scriptEngine.scriptCharacterSet = argv[fileIdx + 1];
                        else {
                            System.err.println(ScriptEngine.getResString("CharactersetIsNotSpecified"));
                            return;
                        }
                        fileIdx += 2;
                    } else {
                        System.err.println(ScriptEngine.getResString("UnknownOption", argv[fileIdx]));
                        return;
                    }
                } else {
                    //System.out.println("-----------------------------------------------------");
                    //System.out.println("run : " + argv[fileIdx]);
                    //scriptEngine.runScriptFile(argv[fileIdx], true);
                    scriptEngine.runScriptFile(argv[fileIdx], false);
                    fileIdx++;
                }
            }
        }
    }
}
