package jp.gr.java_conf.matsubara.interpreter.statement;

import java.util.Map;

import jp.gr.java_conf.matsubara.expresser.RuntimeData;
import jp.gr.java_conf.matsubara.expresser.exception.ExpException;


/**
 * try Xe[ggłB<br/>
 * 
 * 쐬: 2005/01/16
 * 
 * @author m.matsubara
 */
public class TryStatement implements IStatement {
    private IStatement tryStatement = null;
    private Integer catchObjectId = null;
    private IStatement catchStatement = null;
    private IStatement finallyStatement = null;
    
    public TryStatement(IStatement tryStatement, Integer catchObjectId, IStatement catchStatement, IStatement finallyStatement) {
        this.tryStatement = tryStatement;
        this.catchObjectId = catchObjectId;
        this.catchStatement = catchStatement;
        this.finallyStatement = finallyStatement;
    }
    
    
    /*
     *  ( Javadoc)
     * @see jp.gr.java_conf.matsubara.interpreter.statement.IStatement#isUsingVariable(Integer id)
     */
    public int isUsingVariable(Integer id) throws ExpException {
        int result = 0;
        if (tryStatement != null)
            result += tryStatement.isUsingVariable(id);
        if (catchStatement != null)
            result += catchStatement.isUsingVariable(id);
        if (finallyStatement != null)
            result += finallyStatement.isUsingVariable(id);
        return result;
    }

    /*
     *  ( Javadoc)
     * @see jp.gr.java_conf.matsubara.interpreter.statement.IStatement#execute(jp.gr.java_conf.matsubara.expresser.RuntimeData)
     */
    public StatementResult execute(RuntimeData runtime) throws ExpException {
        StatementResult receive = null;
        try {
            receive = tryStatement.execute(runtime);
            if (receive != null) {
                int statementType = receive.getStatementType();
                if (statementType == StatementResult.ST_EXCEPTION) {
                    try {
                        runtime.setValue(catchObjectId, receive.getResultValue());
                        receive = catchStatement.execute(runtime);
                        if (receive != null) {
                            return receive;
                        }
                    } finally {
                    }
                    return null;
                }
                return receive;
            }
        } finally {
            if (finallyStatement != null) {
                receive = finallyStatement.execute(runtime);
                if (receive != null) {
                    return receive;
                }
            }
        }
        return null;
    }
    
    /* ( Javadoc)
     * @see jp.gr.java_conf.matsubara.interpreter.statement.IStatement#registLocalVariable(java.util.Map)
     */
    public void registLocalVariable(Map varMap) {
        if (tryStatement != null)
            tryStatement.registLocalVariable(varMap);
        if (catchStatement != null)
            catchStatement.registLocalVariable(varMap);
        if (finallyStatement != null)
            finallyStatement.registLocalVariable(varMap);
        if (catchObjectId != null) {
            varMap.put(catchObjectId, this);    // o^l͉łǂ
        }
    }
    
}
