package jp.gr.java_conf.matsubara.interpreter.statement;

import java.util.Map;

import jp.gr.java_conf.matsubara.expresser.IVariable;
import jp.gr.java_conf.matsubara.expresser.RuntimeData;
import jp.gr.java_conf.matsubara.expresser.exception.ExpException;
import jp.gr.java_conf.matsubara.expresser.parsedexpression.IParsedExpression;
import jp.gr.java_conf.matsubara.expresser.type.IExpObject;


/**
 * ϐ`Xe[ggłB<br/>
 * 
 * 쐬: 2005/03/06
 * 
 * @author m.matsubara
 */
public class VariableDefineStatement implements IStatement {
    private Integer id;
    private int variableOffset = 0;
    private IParsedExpression initialExpression;
    
    public VariableDefineStatement(Integer id, IParsedExpression initialExpression) {
        this.id = id;
        this.initialExpression = initialExpression;
    }
    

    /* ( Javadoc)
     * @see jp.gr.java_conf.matsubara.interpreter.statement.IStatement#isUsingVariable(java.lang.Integer)
     */
    public int isUsingVariable(Integer id) throws ExpException {
        if (initialExpression != null)
            return initialExpression.isUsingVariable(id);
        else
            return 0;
    }

    /*
     *  ( Javadoc)
     * @see jp.gr.java_conf.matsubara.interpreter.statement.IStatement#execute(jp.gr.java_conf.matsubara.expresser.RuntimeData)
     */
    public StatementResult execute(RuntimeData runtime) throws ExpException {
        if (initialExpression != null) {
            if (variableOffset == 0)
                variableOffset = runtime.getVariableOffset(id);
            
            IExpObject value = initialExpression.evaluate(runtime);
            if (value instanceof IVariable)
                value = ((IVariable)value).getValue();
            if (variableOffset != 0)
                runtime.setValue(id, variableOffset, value);
            else
                runtime.setValue(id, value);
        }
        return null;
    }

    /* ( Javadoc)
     * @see jp.gr.java_conf.matsubara.interpreter.statement.IStatement#registLocalVariable(java.util.Map)
     */
    public void registLocalVariable(Map varMap) {
        varMap.put(id, this);
    }
}
